# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2020 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Configuration for Theo
"""

from rattail.config import ConfigExtension


class TheoConfig(ConfigExtension):
    """
    Rattail config extension for Theo
    """
    key = 'theo'

    def configure(self, config):

        # call the app "Theo" by default
        config.setdefault('rattail', 'app_title', "Theo")

        # Theo comes with its own menu for web app
        config.setdefault('tailbone', 'menus', 'theo.web.menus')

        # do we integrate w/ Catapult?
        if integrate_catapult(config):
            config.setdefault('rattail', 'model', 'theo.db.model_catapult')
            config.setdefault('rattail.importing', 'versions.handler', 'theo.importing.versions_catapult:FromTheoToTheoVersions')

        # do we integrate w/ CORE-POS?
        elif integrate_corepos(config):
            config.setdefault('rattail', 'model', 'theo.db.model_corepos')
            config.setdefault('rattail.importing', 'versions.handler', 'theo.importing.versions_corepos:FromTheoToTheoVersions')


def integrate_catapult(config):
    return config.getbool('theo', 'integrate_catapult', default=False,
                          usedb=False)


def integrate_corepos(config):
    return config.getbool('theo', 'integrate_corepos', default=False,
                          usedb=False)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2020 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Common views
"""

from tailbone.views import common as base

import theo
from theo.config import integrate_catapult, integrate_corepos


class CommonView(base.CommonView):

    project_title = "tailbone-theo"
    project_version = theo.__version__

    def get_packages(self):
        packages = super(CommonView, self).get_packages()

        if integrate_catapult(self.rattail_config):
            import onager
            import rattail_onager
            import tailbone_onager
            packages['onager'] = onager.__version__
            packages['rattail-onager'] = rattail_onager.__version__
            packages['tailbone-onager'] = tailbone_onager.__version__

        elif integrate_corepos(self.rattail_config):
            import corepos
            import rattail_corepos
            import tailbone_corepos
            packages['pyCOREPOS'] = corepos.__version__
            packages['rattail-corepos'] = rattail_corepos.__version__
            packages['tailbone-corepos'] = tailbone_corepos.__version__

        return packages


def includeme(config):
    CommonView.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2020 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Views
"""

from theo.config import integrate_catapult, integrate_corepos


def includeme(config):
    rattail_config = config.registry.settings.get('rattail_config')

    # core views
    config.include('theo.web.views.common')
    config.include('tailbone.views.auth')
    config.include('tailbone.views.tables')
    config.include('tailbone.views.upgrades')
    config.include('tailbone.views.progress')

    # main table views
    config.include('tailbone.views.customergroups')
    config.include('tailbone.views.custorders')
    config.include('tailbone.views.datasync')
    config.include('tailbone.views.email')
    config.include('tailbone.views.messages')
    config.include('tailbone.views.reportcodes')
    config.include('tailbone.views.roles')
    config.include('tailbone.views.settings')
    config.include('tailbone.views.users')

    # do we integrate w/ Catapult?
    if integrate_catapult(rattail_config):
        config.include('tailbone_onager.views.stores')
        config.include('tailbone_onager.views.customers')
        config.include('tailbone.views.members')
        config.include('tailbone_onager.views.employees')
        config.include('tailbone.views.people')
        config.include('tailbone_onager.views.taxes')
        config.include('tailbone_onager.views.departments')
        config.include('tailbone.views.subdepartments')
        config.include('tailbone_onager.views.brands')
        config.include('tailbone_onager.views.vendors')
        config.include('tailbone_onager.views.products')
        config.include('tailbone_onager.views.catapult')

    # do we integrate w/ CORE-POS?
    elif integrate_corepos(rattail_config):
        config.include('tailbone.views.stores')
        config.include('tailbone_corepos.views.customers')
        config.include('tailbone_corepos.views.members')
        config.include('tailbone.views.employees')
        config.include('tailbone_corepos.views.people')
        config.include('tailbone.views.taxes')
        config.include('tailbone_corepos.views.departments')
        config.include('tailbone_corepos.views.subdepartments')
        config.include('tailbone.views.brands')
        config.include('tailbone_corepos.views.vendors')
        config.include('tailbone_corepos.views.products')
        config.include('tailbone_corepos.views.corepos')

    else: # no POS integration
        config.include('tailbone.views.stores')
        config.include('tailbone.views.customers')
        config.include('tailbone.views.members')
        config.include('tailbone.views.employees')
        config.include('tailbone.views.people')
        config.include('tailbone.views.taxes')
        config.include('tailbone.views.departments')
        config.include('tailbone.views.subdepartments')
        config.include('tailbone.views.brands')
        config.include('tailbone.views.vendors')
        config.include('tailbone.views.products')



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2020 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Web Menus
"""

from theo.config import integrate_catapult, integrate_corepos


def simple_menus(request):
    url = request.route_url
    rattail_config = request.rattail_config

    include_catapult = integrate_catapult(rattail_config)
    include_corepos = integrate_corepos(rattail_config)

    orders_menu = {
        'title': "Orders",
        'type': 'menu',
        'items': [
            {
                'title': "New Customer Order",
                'url': url('custorders.create'),
                'perm': 'custorders.create',
            },
            {
                'title': "All New Orders",
                'url': url('new_custorders'),
                'perm': 'new_custorders.list',
            },
            {'type': 'sep'},
            {
                'title': "All Customer Orders",
                'url': url('custorders'),
                'perm': 'custorders.list',
            },
            {
                'title': "All Order Items",
                'url': url('custorders.items'),
                'perm': 'custorders.items.list',
            },
            # {
            #     'title': "Configuration",
            #     'url': url('custorders.config'),
            #     'perm': 'custorders.configure',
            # },
        ],
    }

    people_menu = {
        'title': "People",
        'type': 'menu',
        'items': [
            {
                'title': "Customers",
                'url': url('customers'),
                'perm': 'customers.list',
            },
            {
                'title': "Members",
                'url': url('members'),
                'perm': 'members.list',
            },
            {
                'title': "Employees",
                'url': url('employees'),
                'perm': 'employees.list',
            },
            {
                'title': "All People",
                'url': url('people'),
                'perm': 'people.list',
            },
        ],
    }

    products_menu = {
        'title': "Products",
        'type': 'menu',
        'items': [
            {
                'title': "Products",
                'url': url('products'),
                'perm': 'products.list',
            },
            {
                'title': "Departments",
                'url': url('departments'),
                'perm': 'departments.list',
            },
            {
                'title': "Subdepartments",
                'url': url('subdepartments'),
                'perm': 'subdepartments.list',
            },
            {
                'title': "Brands",
                'url': url('brands'),
                'perm': 'brands.list',
            },
            {
                'title': "Vendors",
                'url': url('vendors'),
                'perm': 'vendors.list',
            },
            {
                'title': "Taxes",
                'url': url('taxes'),
                'perm': 'taxes.list',
            },
        ],
    }

    if include_catapult:
        catapult_menu = {
            'title': "Catapult",
            'type': 'menu',
            'items': [
                {
                    'title': "Departments",
                    'url': url('catapult.departments'),
                    'perm': 'catapult.departments.list',
                },
                {
                    'title': "Terminals",
                    'url': url('catapult.terminals'),
                    'perm': 'catapult.terminals.list',
                },
                {
                    'title': "Transactions",
                    'url': url('catapult.transactions'),
                    'perm': 'catapult.transactions.list',
                },
                {
                    'title': "Worksheets",
                    'url': url('catapult.worksheets'),
                    'perm': 'catapult.worksheets.list',
                },
                {'type': 'sep'},
                {
                    'title': "Security Profiles",
                    'url': url('catapult.security_profiles'),
                    'perm': 'catapult.security_profiles.view',
                },
                {
                    'title': "Master Functions",
                    'url': url('catapult.master_functions'),
                    'perm': 'catapult.master_functions.view',
                },
            ],
        }

    if include_corepos:
        from tailbone_corepos.menus import make_corepos_menu
        corepos_menu = make_corepos_menu(request)

    admin_menu = {
        'title': "Admin",
        'type': 'menu',
        'items': [
            {
                'title': "Stores",
                'url': url('stores'),
                'perm': 'stores.list',
            },
            {
                'title': "Users",
                'url': url('users'),
                'perm': 'users.list',
            },
            {
                'title': "User Events",
                'url': url('userevents'),
                'perm': 'userevents.list',
            },
            {
                'title': "Roles",
                'url': url('roles'),
                'perm': 'roles.list',
            },
            {'type': 'sep'},
            {
                'title': "App Settings",
                'url': url('appsettings'),
                'perm': 'settings.list',
            },
            {
                'title': "Email Settings",
                'url': url('emailprofiles'),
                'perm': 'emailprofiles.list',
            },
            {
                'title': "Email Attempts",
                'url': url('email_attempts'),
                'perm': 'email_attempts.list',
            },
            {
                'title': "Raw Settings",
                'url': url('settings'),
                'perm': 'settings.list',
            },
            {'type': 'sep'},
            {
                'title': "DataSync Changes",
                'url': url('datasyncchanges'),
                'perm': 'datasync.list',
            },
            {
                'title': "Tables",
                'url': url('tables'),
                'perm': 'tables.list',
            },
            {
                'title': "Theo Upgrades",
                'url': url('upgrades'),
                'perm': 'upgrades.list',
            },
        ],
    }

    menus = [
        orders_menu,
        people_menu,
        products_menu,
    ]

    if include_catapult:
        menus.append(catapult_menu)
    if include_corepos:
        menus.append(corepos_menu)

    menus.append(admin_menu)

    return menus



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2020 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Theo setup script
"""

import os
from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))
exec(open(os.path.join(here, 'theo', '_version.py')).read())
README = open(os.path.join(here, 'README.rst')).read()


requires = [
    #
    # Version numbers within comments below have specific meanings.
    # Basically the 'low' value is a "soft low," and 'high' a "soft high."
    # In other words:
    #
    # If either a 'low' or 'high' value exists, the primary point to be
    # made about the value is that it represents the most current (stable)
    # version available for the package (assuming typical public access
    # methods) whenever this project was started and/or documented.
    # Therefore:
    #
    # If a 'low' version is present, you should know that attempts to use
    # versions of the package significantly older than the 'low' version
    # may not yield happy results.  (A "hard" high limit may or may not be
    # indicated by a true version requirement.)
    #
    # Similarly, if a 'high' version is present, and especially if this
    # project has laid dormant for a while, you may need to refactor a bit
    # when attempting to support a more recent version of the package.  (A
    # "hard" low limit should be indicated by a true version requirement
    # when a 'high' version is present.)
    #
    # In any case, developers and other users are encouraged to play
    # outside the lines with regard to these soft limits.  If bugs are
    # encountered then they should be filed as such.
    #
    # package                           # low                   high

    # TODO: must cap this for now, b/c it breaks Catapult integration?!
    # (something about "Syntax error near 'ROWS'" with grid queries)
    'SQLAlchemy<1.3',                   #                       1.2.19

    'invoke',                           # 1.4.1
    'psycopg2',                         # 2.8.5
    'rattail[db,auth,bouncer]',         # 0.9.130
    'Tailbone',                         # 0.8.97
]


extras = {

    'catapult': [
        #
        # package                       # low                   high

        'onager',                       # 0.2.8
        'rattail-onager',               # 0.2.1
        'tailbone-onager',              # 0.2.3
    ],

    'corepos': [
        #
        # package                       # low                   high

        # TODO: must cap this for now, b/c it breaks CORE-POS integration?!
        # (sometimes there are segfaults with basic grid queries)
        'mysql-connector-python==8.0.17',

        'pyCOREPOS',                    # 0.1.0
        'rattail-corepos',              # 0.1.0
        'tailbone-corepos',             # 0.1.1
    ],
}


setup(
    name = "tailbone-theo",
    version = __version__,
    author = "Lance Edgar",
    author_email = "lance@edbob.org",
    url = "https://rattailproject.org",
    description = "Theo, the order system",
    long_description = README,

    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Framework :: Pyramid',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Office/Business',
    ],

    install_requires = requires,
    extras_require = extras,
    packages = find_packages(),
    include_package_data = True,

    entry_points = {

        'rattail.config.extensions': [
            'theo = theo.config:TheoConfig',
        ],

        'paste.app_factory': [
            'main = theo.web.app:main',
        ],
    },
)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2020 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Theo web app
"""

from tailbone import app

from theo.config import integrate_catapult, integrate_corepos


def main(global_config, **settings):
    """
    This function returns a Pyramid WSGI application.
    """
    # prefer Theo templates over Tailbone
    settings.setdefault('mako.directories', ['theo.web:templates',
                                             'tailbone:templates',])

    # for graceful handling of postgres restart
    settings.setdefault('retry.attempts', 2)

    # make config objects
    rattail_config = app.make_rattail_config(settings)
    pyramid_config = app.make_pyramid_config(settings)

    # maybe configure integration db connections
    if integrate_catapult(rattail_config):
        from tailbone_onager.db import CatapultSession
        CatapultSession.configure(bind=rattail_config.catapult_engine)
    if integrate_corepos(rattail_config):
        from tailbone_corepos.db import CoreOfficeSession
        CoreOfficeSession.configure(bind=rattail_config.corepos_engine)

    # bring in the rest of Theo
    pyramid_config.include('theo.web.static')
    pyramid_config.include('theo.web.subscribers')
    pyramid_config.include('theo.web.views')

    # for graceful handling of postgres restart
    pyramid_config.add_tween('tailbone.tweens.sqlerror_tween_factory',
                             under='pyramid_tm.tm_tween_factory')

    return pyramid_config.make_wsgi_app()
